var assert = require('assert'),
    path = require('path'),
    underscore = require('underscore'),
    JSLINT = require('JSLint-commonJS'),
    fs = require('fs');

var walkTree = function(root, fn) {
    var filenameParts, extension, path, stat;
    var files = fs.readdirSync(root);
    files.forEach(function(file) {
        path = root + '/' + file;
        stat = fs.lstatSync(path);
        fn(path, stat);
        if (stat.isDirectory() && !stat.isSymbolicLink()) {
            walkTree(path, fn);
        }
    });
};

var getFilePaths = function(roots) {
    if (!_.isArray(roots)) {
        roots = [roots];
    }
    var paths = [];
    roots.forEach(function(root) {
        walkTree(root, function(file, stat) {
            filenameParts = file.split('.');
            extension = filenameParts[filenameParts.length - 1];
            if (extension == 'js' || extension == 'json') {
                paths.push(file);
            }
        });
    });
    return paths;
};

assertLintFree = exports.assertLintFree = function (root, options) {
    var paths = getFilePaths(root);
    var errors = {};
    paths.forEach(function(path) {
        var source = fs.readFileSync(path, 'utf8');
        var lintFree = JSLINT(source, options);
        if (!lintFree) {
            errors[path] = JSLINT.errors;
        }
    });
    Object.keys(errors).forEach(function(path) {
        errors[path].forEach(function(error) {
            // option to ignore an array of reasons
            var allowAssert = true;
            if (options && options.ignore) {
                options.ignore.forEach(function(reason) {
                    if (error && reason === error.reason) {
                        allowAssert = false;
                    }
                });
            }
            if (error && allowAssert) {
                assert.ok(
                    false, 
                    "\n\tProblem at line " + error.line + " character " + error.character + ": " + error.reason + "\n\tFile: " + path
                );
            }
        });
    });
};

module.exports = assertLintFree;
